@@ -1,8 +1,8 @@
-/*  $Id: BurgerSpaceEngine.cpp,v 1.11 2003/01/11 01:41:15 sarrazip Exp $
+/*  $Id: BurgerSpaceEngine.cpp,v 1.18 2004/05/02 04:55:47 sarrazip Exp $
     BurgerSpaceEngine.cpp - Main engine
 
     burgerspace - A hamburger-smashing video game.
-    Copyright (C) 2001 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2001-2004 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -24,30 +24,9 @@
 
 #include "EnemySprite.h"
 
-#include <gengameng/PixmapLoadError.h>
+#include <flatzebra/PixmapLoadError.h>
 
-#include <stdlib.h>
-#include <fstream>
-#include <stdio.h>
-#include <string.h>
-#include <ctype.h>
-#include <math.h>
-#include <stdarg.h>
-
-#include <sys/time.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#include <unistd.h>
-#include <errno.h>
-
-#include <map>
-#include <vector>
-#include <algorithm>
-#include <iomanip>
-
-using namespace std;
+#include <assert.h>
 
 
 /*  Tiles:
@@ -168,6 +147,7 @@ using namespace std;
 #include "images/digit8.xpm"
 #include "images/digit9.xpm"
 
+using namespace std;
 
 #define __(s) (s)  /* Gettext-like macro */
 
@@ -199,14 +179,16 @@ static const int
 // UTILITY FUNCTIONS
 //
 
-inline int
+inline
+int
 Rnd(int lowerLimit, int upperLimit)
 {
     return rand() % (upperLimit - lowerLimit + 1) + lowerLimit;
 }
 
 
-static void
+static
+void
 removeNullElementsFromSpriteList(SpriteList &slist)
 {
     SpriteList::iterator it =
@@ -221,14 +203,16 @@ removeNullElementsFromSpriteList(SpriteList &slist)
 }
 
 
-static void
+static
+void
 deleteSprite(Sprite *p)
 {
     delete p;
 }
 
 
-static void
+static
+void
 deleteIngredientGroup(IngredientGroup *p)
 {
     delete p;
@@ -236,7 +220,8 @@ deleteIngredientGroup(IngredientGroup *p)
 
 
 template <class Container>
-static void
+static
+void
 deleteSprites(Container &c)
 {
     for_each(c.begin(), c.end(), deleteSprite);
@@ -244,7 +229,8 @@ deleteSprites(Container &c)
 }
 
 
-static string
+static
+string
 itoa(unsigned long integer)
 {
     char buffer[256];
@@ -253,7 +239,8 @@ itoa(unsigned long integer)
 }
 
 
-static Couple
+static
+Couple
 getCoupleFromDirection(int direction, int amplitude)
 {
     switch (direction)
@@ -267,7 +254,8 @@ getCoupleFromDirection(int direction, int amplitude)
 }
 
 
-static int
+static
+int
 getDirectionFromCouple(Couple v)
 {
     assert((v.x != 0) ^ (v.y != 0));  // either x or y must be zero, not both
@@ -278,7 +266,8 @@ getDirectionFromCouple(Couple v)
 }
 
 
-int makeDivisibleByInt(int n, int divisor)
+int
+makeDivisibleByInt(int n, int divisor)
 /*  Round 'n' to the lowest multiple of divisor.
     Works also if 'n' is negative.
 */
@@ -804,8 +793,8 @@ BurgerSpaceEngine::Level::init(int no, int nCols, int nRows, Couple posInPixels)
     memset(xpmMatrix, 0, nRows * nCols * sizeof(XPM));
 
     delete [] tileMatrix;
-    tileMatrix = new Pixmap[nRows * nCols];
-    memset(tileMatrix, 0, nRows * nCols * sizeof(Pixmap));
+    tileMatrix = new SDL_Surface *[nRows * nCols];
+    memset(tileMatrix, 0, nRows * nCols * sizeof(SDL_Surface *));
 }
 
 
@@ -827,7 +816,7 @@ BurgerSpaceEngine::Level::getLevelNo() const
 
 void
 BurgerSpaceEngine::Level::setTileMatrixEntry(
-			int colNo, int rowNo, XPM xpm, Pixmap pixmap)
+			int colNo, int rowNo, XPM xpm, SDL_Surface *pixmap)
 {
     assert(colNo < sizeInTiles.x);
     assert(rowNo < sizeInTiles.y);
@@ -838,14 +827,14 @@ BurgerSpaceEngine::Level::setTileMatrixEntry(
 }
 
 
-inline BurgerSpaceEngine::Pixmap *
+inline SDL_Surface **
 BurgerSpaceEngine::Level::getTileMatrixRow(int rowNo)
 {
     return &tileMatrix[rowNo * sizeInTiles.x];
 }
 
 
-XPM
+BurgerSpaceEngine::XPM
 BurgerSpaceEngine::Level::getXPMAtPixel(Couple pos) const
 /*  Returns a pointer to the XPM variable that corresponds to
     the tile that contains the pixels at 'pos'.
@@ -868,7 +857,9 @@ BurgerSpaceEngine::Level::getXPMAtPixel(Couple pos) const
 BurgerSpaceEngine::BurgerSpaceEngine(const string &windowManagerCaption,
 					int initLevelNumber,
 					bool useSound,
-					bool fullScreen)
+					bool fullScreen,
+					bool useOldMotionMode,
+					SDLKey pepperKey)
 						throw(int, string)
   : GameEngine(Couple(672, 520), windowManagerCaption, fullScreen),
   		// may throw string exception
@@ -883,6 +874,7 @@ BurgerSpaceEngine::BurgerSpaceEngine(const string &windowManagerCaption,
     playerPA(18),
     playerSprite(NULL),
     lastPlayerDirection(-1),
+    oldMotionMode(useOldMotionMode),
 
     pepperPA(2),
     pepperSprites(),
@@ -910,17 +902,14 @@ BurgerSpaceEngine::BurgerSpaceEngine(const string &windowManagerCaption,
     digitPA(10),
     scoreSprites(),
 
-    escapePressed(false),
-    leftArrowPressed(false),
-    rightArrowPressed(false),
-    upArrowPressed(false),
-    downArrowPressed(false),
-    spacePressed(false),
-    spacePressedBefore(false),
-    ctrlPressed(false),
-    ctrlPressedBefore(false),
-    letterPPressed(false),
-    letterPPressedBefore(false),
+    quitKS(SDLK_ESCAPE),
+    leftKS(SDLK_LEFT),
+    rightKS(SDLK_RIGHT),
+    upKS(SDLK_UP),
+    downKS(SDLK_DOWN),
+    startKS(SDLK_SPACE),
+    shootKS(pepperKey, SDLK_RCTRL),
+    pauseKS(SDLK_p),
 
     numHamburgersToDo(0),
     thePeakScore(0),
@@ -1084,9 +1073,18 @@ BurgerSpaceEngine::initializeMisc() throw(string)
     {
 	try
 	{
-	    string d = getDir(PKGSOUNDDIR, "PKGSOUNDDIR");
+	    theSoundMixer = NULL;
 	    theSoundMixer = new SoundMixer(16);  // may throw string
+	}
+	catch (const SoundMixer::Error &e)
+	{
+	    return;
+	}
 
+	string d = getDir(PKGSOUNDDIR, "PKGSOUNDDIR");
+
+	try
+	{
 	    ingredientBouncesSound.init(d + "ingredient-bounces.wav");
 	    ingredientInPlateSound.init(d + "ingredient-in-plate.wav");
 	    ingredientFallsSound  .init(d + "ingredient-falls.wav");
@@ -1135,40 +1133,33 @@ BurgerSpaceEngine::initGameParameters()
     updateNumLives = true;
 
     initTimeForTreat();
-
-    #if DIRT_MANAGEMENT
-    displayStartMessage(false);
-    #endif
 }
 
 
 ///////////////////////////////////////////////////////////////////////////////
 
 
-/*virtual*/ void
+/*virtual*/
+void
 BurgerSpaceEngine::processKey(SDLKey keysym, bool pressed)
 {
-    switch (keysym)
-    {
-	case SDLK_ESCAPE:  escapePressed = pressed; break;
-	case SDLK_LEFT:    leftArrowPressed = pressed; break;
-	case SDLK_RIGHT:   rightArrowPressed = pressed; break;
-	case SDLK_UP:      upArrowPressed = pressed; break;
-	case SDLK_DOWN:    downArrowPressed = pressed; break;
-	case SDLK_SPACE:   spacePressed = pressed; break;
-	case SDLK_LCTRL:   ctrlPressed = pressed; break;
-	case SDLK_RCTRL:   ctrlPressed = pressed; break;
-	case SDLK_p:       letterPPressed = pressed; break;
-
-	default:           ;  // ignore other keys
-    }
+    quitKS.check(keysym, pressed);
+    leftKS.check(keysym, pressed);
+    rightKS.check(keysym, pressed);
+    upKS.check(keysym, pressed);
+    downKS.check(keysym, pressed);
+    startKS.check(keysym, pressed);
+    shootKS.check(keysym, pressed);
+    shootKS.check(keysym, pressed);
+    pauseKS.check(keysym, pressed);
 }
 
 
-/*virtual*/ bool
+/*virtual*/
+bool
 BurgerSpaceEngine::tick()
 {
-    if (escapePressed)
+    if (quitKS.isPressed())
 	return false;
 
     if (paused)
@@ -1176,14 +1167,8 @@ BurgerSpaceEngine::tick()
 	restoreBackground();
 	drawSprites();
 	displayPauseMessage(true);
-	if (letterPPressed && !letterPPressedBefore)
-	{
+	if (pauseKS.justPressed())
 	    paused = false;
-
-	    #if DIRT_MANAGEMENT
-	    displayPauseMessage(false);
-	    #endif
-	}
     }
     else
     {
@@ -1198,10 +1183,10 @@ BurgerSpaceEngine::tick()
 	detectCollisions();
     }
 
-    spacePressedBefore = spacePressed;
-    ctrlPressedBefore = ctrlPressed;
-    letterPPressedBefore = letterPPressed;
-    
+    startKS.remember();
+    shootKS.remember();
+    pauseKS.remember();
+
     return true;
 }
 
@@ -1227,10 +1212,6 @@ void
 BurgerSpaceEngine::putSprite(const Sprite &s)
 {
     copySpritePixmap(s, s.currentPixmapIndex, s.getPos());
-
-    #if DIRT_MANAGEMENT
-    theCurrentLevel.takeSpriteDirt(s);
-    #endif
 }
 
 
@@ -1280,13 +1261,31 @@ BurgerSpaceEngine::startNewLife()
 }
 
 
-bool
-BurgerSpaceEngine::isPepperRequested() const
-/*  Indicates if the chef must shoot.
-*/
+static const struct
 {
-    return ctrlPressed && !ctrlPressedBefore;
+    unsigned char num;      // number of alternatives
+    unsigned char alts[4];  // alternatives for the key
 }
+directionTable[16] =
+{
+    {  0, {  0,  0,  0,  0 } },    //  0
+    {  1, {  1,  0,  0,  0 } },    //  1
+    {  1, {  2,  0,  0,  0 } },    //  2
+    {  2, {  1,  2,  0,  0 } },    //  3
+    {  1, {  4,  0,  0,  0 } },    //  4
+    {  2, {  4,  1,  0,  0 } },    //  5
+    {  2, {  4,  2,  0,  0 } },    //  6
+    {  3, {  4,  2,  1,  0 } },    //  7
+    {  1, {  8,  0,  0,  0 } },    //  8
+    {  2, {  8,  1,  0,  0 } },    //  9
+    {  2, {  8,  2,  0,  0 } },    // 10
+    {  3, {  8,  2,  1,  0 } },    // 11
+    {  2, {  8,  4,  0,  0 } },    // 12
+    {  3, {  8,  4,  1,  0 } },    // 13
+    {  3, {  8,  4,  2,  0 } },    // 14
+    {  4, {  8,  4,  2,  1 } },    // 15
+};
+
 
 
 void
@@ -1299,31 +1298,6 @@ BurgerSpaceEngine::chooseDirectionAmongMany(bool directions[4]) const
     The choice is random.
 */
 {
-    static const struct
-    {
-	unsigned char num;      // number of alternatives
-	unsigned char alts[4];  // alternatives for the key
-    }
-    table[16] =
-    {
-	{  0, {  0,  0,  0,  0 } },    //  0
-	{  1, {  1,  0,  0,  0 } },    //  1
-	{  1, {  2,  0,  0,  0 } },    //  2
-	{  2, {  1,  2,  0,  0 } },    //  3
-	{  1, {  4,  0,  0,  0 } },    //  4
-	{  2, {  4,  1,  0,  0 } },    //  5
-	{  2, {  4,  2,  0,  0 } },    //  6
-	{  3, {  4,  2,  1,  0 } },    //  7
-	{  1, {  8,  0,  0,  0 } },    //  8
-	{  2, {  8,  1,  0,  0 } },    //  9
-	{  2, {  8,  2,  0,  0 } },    // 10
-	{  3, {  8,  2,  1,  0 } },    // 11
-	{  2, {  8,  4,  0,  0 } },    // 12
-	{  3, {  8,  4,  1,  0 } },    // 13
-	{  3, {  8,  4,  2,  0 } },    // 14
-	{  4, {  8,  4,  2,  1 } },    // 15
-    };
-
     unsigned char key =
 	    ( (directions[RIGHT] << RIGHT)
 	    | (directions[UP]    << UP)
@@ -1332,9 +1306,9 @@ BurgerSpaceEngine::chooseDirectionAmongMany(bool directions[4]) const
 	    );
     if (key == 0)
 	return;
-    unsigned char num = table[key].num;
+    unsigned char num = directionTable[key].num;
     unsigned char index = rand() % num;
-    unsigned char choice = table[key].alts[index];
+    unsigned char choice = directionTable[key].alts[index];
 
     directions[RIGHT] = ((choice & (1 << RIGHT)) != 0);
     directions[UP]    = ((choice & (1 << UP))    != 0);
@@ -1426,10 +1400,10 @@ BurgerSpaceEngine::getPlayerDesiredDirections(bool desiredDirs[4]) const
     allow us to have the chef be controlled by the computer.
 */
 {
-    desiredDirs[RIGHT] = rightArrowPressed;
-    desiredDirs[UP]    = upArrowPressed;
-    desiredDirs[LEFT]  = leftArrowPressed;
-    desiredDirs[DOWN]  = downArrowPressed;
+    desiredDirs[RIGHT] = rightKS.isPressed();
+    desiredDirs[UP]    = upKS.isPressed();
+    desiredDirs[LEFT]  = leftKS.isPressed();
+    desiredDirs[DOWN]  = downKS.isPressed();
 }
 
 
@@ -1438,7 +1412,7 @@ BurgerSpaceEngine::animatePlayer()
 /*  Returns true if the game must continue, or false to have it stop.
 */
 {
-    if (letterPPressed && !letterPPressedBefore)
+    if (pauseKS.justPressed())
     {
 	paused = true;
 	return true;
@@ -1491,11 +1465,6 @@ BurgerSpaceEngine::animatePlayer()
 	    {
 		addToNumLives(-1);
 		assert(numLives >= 0);
-		if (numLives == 0)
-		{
-		    initNextLevel(initLevelNo);
-		    //showInstructions();
-		}
 	    }
 
 	}
@@ -1506,18 +1475,15 @@ BurgerSpaceEngine::animatePlayer()
 
     if (numLives == 0)
     {
-	//showInstructions();
-
 	/*  In the demo mode, the Space bar starts a new game.
-	    (But, in a network game, the user who does not control the chef
-	    waits for the other user to start the game.)
 	*/
-	if (!spacePressed && spacePressedBefore)
+	if (startKS.justPressed())
 	{
 	    playSoundEffect(newGameStartsSound);
 
 	    initGameParameters();
 	    startNewLife();
+	    initNextLevel(initLevelNo);
 
 	    theScore = 0;
 	    thePeakScore = 0;
@@ -1537,9 +1503,9 @@ BurgerSpaceEngine::animatePlayer()
     /*  Shoot if requested:
     */
     static const bool infinitePepper = (getenv("INFINITEPEPPER") != NULL);
-    if (isPepperRequested())
+    if (shootKS.justPressed())
     {
-	if ((infinitePepper || numAvailablePeppers > 0) && isPepperRequested())
+	if (infinitePepper || numAvailablePeppers > 0)
 	{
 	    const Couple size = pepperPA.getImageSize();
 	    const Couple plsize = playerSprite->getSize();
@@ -1588,7 +1554,29 @@ BurgerSpaceEngine::animatePlayer()
 
     playerSprite->currentPixmapIndex = 0;
     if (playerSpeed.isZero())
-        return true;
+    {
+	if (!oldMotionMode
+		&& (desiredDirs[RIGHT] || desiredDirs[UP]
+		    || desiredDirs[LEFT] || desiredDirs[DOWN]))
+	{
+	    /*
+		If the user is pressing at least one arrow, then try to
+		reuse the last non zero direction.
+		This can be is useful to turn corners automatically.
+	    */
+	    bool backupDirs[4] = { false, false, false, false };
+	    backupDirs[lastPlayerDirection] = true;
+	    playerSpeed = attemptMove(*playerSprite,
+				    backupDirs[LEFT], backupDirs[RIGHT],
+				    backupDirs[UP], backupDirs[DOWN],
+				    PLAYER_SPEED_FACTOR);
+	    if (playerSpeed.isZero())
+		return true;
+	}
+	else
+	    return true;
+    }
+
 
     /*  Determine the sprite direction and image according to 
         the player's speed:
@@ -1906,7 +1894,7 @@ BurgerSpaceEngine::animateAutomaticCharacters()
 
     /*  Ingredients:
     */
-    for (IngredientSpriteList::iterator iti = ingredientSprites.begin();
+    for (IngredientSprite::List::iterator iti = ingredientSprites.begin();
 				    iti != ingredientSprites.end(); iti++)
 	(*iti)->addSpeedToPos();
 
@@ -1968,7 +1956,7 @@ BurgerSpaceEngine::findIngredientGroupRightBelow(
     Couple targetPos = upperGroup.getMember(0)->getPos();
     targetPos.y += upperGroup.getMember(0)->getSize().y;
 
-    for (IngredientGroupList::iterator it = ingredientGroups.begin();
+    for (IngredientGroup::List::iterator it = ingredientGroups.begin();
 					    it != ingredientGroups.end(); it++)
     {
 	IngredientGroup *lowerGroup = *it;
@@ -2297,7 +2285,7 @@ BurgerSpaceEngine::releaseAllCarriedEnemies()
     /*  Tell all ingredient groups to release all carried enemies.
 	This must be done before we destroy the enemy sprites.
     */
-    for (IngredientGroupList::iterator it = ingredientGroups.begin();
+    for (IngredientGroup::List::iterator it = ingredientGroups.begin();
     					it != ingredientGroups.end(); it++)
     {
 	assert(*it != NULL);
@@ -2314,7 +2302,7 @@ BurgerSpaceEngine::detectCollisions()
 
     bool playIngredientLoweredSound = false;
 
-    for (IngredientGroupList::iterator it = ingredientGroups.begin();
+    for (IngredientGroup::List::iterator it = ingredientGroups.begin();
 					it != ingredientGroups.end(); it++)
     {
 	IngredientGroup *aGroup = *it;
@@ -2583,7 +2571,7 @@ BurgerSpaceEngine::createScoreSprites(long n, Couple center)
     addToScore(n);
 
     char number[64];
-    sprintf(number, "%ld", n);
+    snprintf(number, sizeof(number), "%ld", n);
     size_t numDigits = strlen(number);
 
     Couple digitSize = digitPA.getImageSize();
@@ -2595,8 +2583,8 @@ BurgerSpaceEngine::createScoreSprites(long n, Couple center)
         int digit = number[i] - '0';
         Sprite *s = new Sprite(digitPA,
 			    scorePos + i * Couple(digitSize.x + 2, 0),
-			    Couple(0, -1), Couple(0, 0),
-			    Couple(0, 0), Couple(0, 0));
+			    Couple(0, -1), Couple(),
+			    Couple(), Couple());
         s->setTimeToLive(SCORE_TTL);
 	s->currentPixmapIndex = digit;
         scoreSprites.push_back(s);
@@ -2738,7 +2726,7 @@ BurgerSpaceEngine::loadPixmaps() throw(PixmapLoadError)
     loadPixmap(fries_xpm,    treatPA, 2);
 
 
-    /*  Scores:
+    /*  Digits:
     */
     loadPixmap(digit0_xpm, digitPA, 0);
     loadPixmap(digit1_xpm, digitPA, 1);
@@ -2803,7 +2791,7 @@ BurgerSpaceEngine::loadLevel(int levelNo) throw(string)
 	for (size_t colNo = 0; levelDesc[rowNo][colNo] != '\0'; colNo++)
 	{
 	    XPM xpm = NULL;
-	    Pixmap pixmap = (Pixmap) 0;
+	    SDL_Surface *pixmap = NULL;
 	    switch (levelDesc[rowNo][colNo])
 	    {
 		case 'e':  xpm = empty_xpm;  pixmap = tilePixmaps[0]; break;
@@ -2830,27 +2818,13 @@ BurgerSpaceEngine::restoreBackground()
 {
     for (int rowNo = 0; rowNo < theCurrentLevel.sizeInTiles.y; rowNo++)
     {
-	Pixmap *pixmapRow = theCurrentLevel.getTileMatrixRow(rowNo);
-
-	#if DIRT_MANAGEMENT
-	bool *dirtRow = theCurrentLevel.getDirtMatrixRow(rowNo);
-	assert(dirtRow != NULL);
-	#endif
+	SDL_Surface **pixmapRow = theCurrentLevel.getTileMatrixRow(rowNo);
 
 	for (int colNo = 0; colNo < theCurrentLevel.sizeInTiles.x; colNo++)
 	{
-	    #if DIRT_MANAGEMENT
-	    if (dirtRow[colNo])
-	    #endif
-	    {
-		Couple pos = theCurrentLevel.positionInPixels +
-				TILE_SIDE * Couple(colNo, rowNo);
-		copyPixmap(pixmapRow[colNo], pos);
-
-		#if DIRT_MANAGEMENT
-		dirtRow[colNo] = false;
-		#endif
-	    }
+	    Couple pos = theCurrentLevel.positionInPixels +
+			    TILE_SIDE * Couple(colNo, rowNo);
+	    copyPixmap(pixmapRow[colNo], pos);
 	}
     }
 }
@@ -2874,7 +2848,7 @@ BurgerSpaceEngine::drawSprites()
 
     /*  Ingredients:
     */
-    for (IngredientSpriteList::const_iterator iti = ingredientSprites.begin();
+    for (IngredientSprite::List::const_iterator iti = ingredientSprites.begin();
 					iti != ingredientSprites.end(); iti++)
 	putSprite(**iti);
 
@@ -2909,9 +2883,8 @@ BurgerSpaceEngine::drawSprites()
 
     /*  Score, number of lives and number of available peppers:
     */
-    #if !DIRT_MANAGEMENT
-    updateScore = updateNumLives = updateNumAvailablePeppers = updateLevelNo = true;
-    #endif
+    updateScore = updateNumLives =
+		updateNumAvailablePeppers = updateLevelNo = true;
     char s[64];
     if (updateScore)
     {
@@ -3033,12 +3006,8 @@ BurgerSpaceEngine::displayPauseMessage(bool display)
     corresponding region if 'display' is false.
 */
 {
-    const char *s = (display
-    			? "PAUSED -- press P to resume"
-    			: "");
-    			//: "                           ");
-    //writeString(s, messagePos);
-    displayMessage(s);
+    if (display)
+	displayMessage(2, "PAUSED -- press P to resume");
 }
 
 
@@ -3048,31 +3017,27 @@ BurgerSpaceEngine::displayStartMessage(bool display)
     corresponding region if 'display' is false.
 */
 {
-    if (!display)
-	displayMessage("");
-    else
+    if (display)
     {
-	static unsigned long firstTick = tickCount;
+	displayMessage(0, "BurgerSpace " VERSION " - by Pierre Sarrazin");
+	displayMessage(2, "Move with arrow keys - shoot pepper with Ctrl key");
+	displayMessage(3, "Press SPACE to start - press Escape to quit");
 
-	if ((tickCount - firstTick) % 64 < 32)
-	    displayMessage("BurgerSpace " VERSION " - by Pierre Sarrazin");
-	else
-	    displayMessage("Press SPACE to start");
+	if (useSound && theSoundMixer == NULL)
+	    displayMessage(5, "*** Sound device not available ***");
     }
 }
 
 
 void
-BurgerSpaceEngine::displayMessage(const char *msg)
+BurgerSpaceEngine::displayMessage(int row, const char *msg)
 {
     static const Couple fontdim = getFontDimensions();
-    static const char *blank = "                                      ";
 
-    int x = (theScreenSizeInPixels.x - strlen(msg) * fontdim.x) / 2;
-    int y = theScreenSizeInPixels.y - 20;
+    const size_t len = strlen(msg);
+    int x = (theScreenSizeInPixels.x - len * fontdim.x) / 2;
+    int y = theScreenSizeInPixels.y + (row - 10) * fontdim.y;
 
-    if (msg != blank)  // first erase the region, then write the string
-	displayMessage(blank);
     writeString(msg, Couple(x, y));
 }
 
