@@ -1,8 +1,8 @@
-/*  $Id: BurgerSpaceEngine.h,v 1.8 2003/01/11 01:41:15 sarrazip Exp $
+/*  $Id: BurgerSpaceEngine.h,v 1.16 2004/05/02 04:55:47 sarrazip Exp $
     BurgerSpaceEngine.h - Main engine
 
     burgerspace - A hamburger-smashing video game.
-    Copyright (C) 2001 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2001-2004 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -26,17 +26,12 @@
 #include "IngredientGroup.h"
 #include "IngredientSprite.h"
 
-#include "SoundMixer.h"
-
-#include <gengameng/GameEngine.h>
-#include <gengameng/Sprite.h>
+#include <flatzebra/GameEngine.h>
+#include <flatzebra/Sprite.h>
+#include <flatzebra/SoundMixer.h>
+#include <flatzebra/KeyState.h>
 
 #include <string>
-#include <list>
-
-using namespace std;
-
-typedef char **XPM;
 
 
 class BurgerSpaceEngine : public GameEngine
@@ -45,8 +40,7 @@ class BurgerSpaceEngine : public GameEngine
 {
 public:
 
-    typedef PixmapArray::Pixmap Pixmap;
-
+    typedef char **XPM;
 
     enum { RIGHT, UP, LEFT, DOWN };
     /*	Possible directions of movement, in trigonometric order.
@@ -54,11 +48,13 @@ public:
 	The order of the constants is guaranteed, so they are 0, 1, 2, 3.
     */
 
-    BurgerSpaceEngine(const string &windowManagerCaption,
+    BurgerSpaceEngine(const std::string &windowManagerCaption,
 			int initLevelNumber,
 			bool useSound,
-			bool fullScreen)
-				throw(int, string);
+			bool fullScreen,
+			bool useOldMotionMode,
+			SDLKey pepperKey)
+				throw(int, std::string);
     /*  See base class.
 
 	'windowManagerCaption' must contain the title to give to the window.
@@ -66,7 +62,15 @@ public:
 	The game will start at level 'initLevelNumber'.
 	This must be a positive integer.
 
-	Throws a string or an integer code if an error occurs.
+	'useOldMotionMode' determines if the pre v1.7.1 player motion
+	should be used, or not.
+	In the new motion system, when the requested direction is impossible,
+	the last moving direction is attempted.
+	This can be useful to turn corners automatically.
+	In the old system, when the requested direction is impossible,
+	the player does not move.
+
+	Throws a std::string or an integer code if an error occurs.
     */
 
     virtual ~BurgerSpaceEngine();
@@ -90,27 +94,6 @@ private:
     //
 
 
-    enum Key
-    {
-	NOKEY = 0,
-
-	// The next four constants are guaranteed to have the values 1 to 4:
-	UP_ARROW,
-	DOWN_ARROW,
-	LEFT_ARROW,
-	RIGHT_ARROW,
-
-	LETTER_P,
-	ENTER,
-	SPACE,
-	CTRL,
-	ALT,
-	ESC,
-
-	NUM_SUPPORTED_KEYS
-    };
-
-
     struct IntPair
     {
 	int first, second;
@@ -137,16 +120,17 @@ private:
 	void setLevelNo(int no);
 	int  getLevelNo() const;
 	void setTileMatrixEntry(int colNo, int rowNo,
-				XPM xpm, Pixmap pixmap);
-	Pixmap *getTileMatrixRow(int rowNo);
+				XPM xpm, SDL_Surface *pixmap);
+	SDL_Surface **getTileMatrixRow(int rowNo);
 	XPM getXPMAtPixel(Couple pos) const;
 
     private:
 	int levelNo;
 	XPM *xpmMatrix;   // array of XPM pointers
-	Pixmap *tileMatrix;  // array of X11 pixmaps
+	SDL_Surface **tileMatrix;  // array of X11 pixmaps
     };
 
+
     class IngInit
     {
     public:
@@ -201,7 +185,7 @@ private:
     bool paused;
     unsigned long tickCount;
 
-    Pixmap tilePixmaps[5];
+    SDL_Surface *tilePixmaps[5];
 
 
     /*  PLAYER:
@@ -211,6 +195,7 @@ private:
     PixmapArray playerPA;
     Sprite *playerSprite;
     int lastPlayerDirection;
+    bool oldMotionMode;
 
     PixmapArray pepperPA;
     SpriteList  pepperSprites;
@@ -236,8 +221,8 @@ private:
     PixmapArray redStuffPA;
     PixmapArray yellowStuffPA;
     PixmapArray bottomBunPA;
-    IngredientSpriteList ingredientSprites;  // owns the contained objects
-    IngredientGroupList  ingredientGroups;
+    IngredientSprite::List ingredientSprites;  // owns the contained objects
+    IngredientGroup::List  ingredientGroups;
 
 
     /*  TREATS (icecream, etc, that the player eats to get a pepper):
@@ -255,14 +240,15 @@ private:
 
     /*  KEYBOARD COMMANDS:
     */
-    bool escapePressed;
-    bool leftArrowPressed;
-    bool rightArrowPressed;
-    bool upArrowPressed;
-    bool downArrowPressed;
-    bool spacePressed, spacePressedBefore;
-    bool ctrlPressed, ctrlPressedBefore;
-    bool letterPPressed, letterPPressedBefore;
+    KeyState quitKS;
+    KeyState leftKS;
+    KeyState rightKS;
+    KeyState upKS;
+    KeyState downKS;
+    KeyState startKS;
+    KeyState shootKS;
+    KeyState pauseKS;
+
 
     int    numHamburgersToDo;
     long   thePeakScore;  // player's best score yet in this game
@@ -315,7 +301,6 @@ private:
     //  IMPLEMENTATION FUNCTIONS
     //
     //
-    bool isPepperRequested() const;
     void chooseDirectionAmongMany(bool directions[4]) const;
     int  chooseDirectionTowardTarget(
 			    Couple startPos, Couple targetPos, int speedFactor,
@@ -346,10 +331,10 @@ private:
 			const Couple groupPos, const Couple groupSize,
 			const Sprite &s) const;
     void loadPixmaps() throw(PixmapLoadError);
-    void loadLevel(int levelNo) throw(string);
-    void displayErrorMessage(const string &msg) throw();
+    void loadLevel(int levelNo) throw(std::string);
+    void displayErrorMessage(const std::string &msg) throw();
     void initializeSprites() throw(PixmapLoadError);
-    void initializeMisc() throw(string);
+    void initializeMisc() throw(std::string);
     void killSpritesInList(SpriteList &sl);
     IngredientGroup *findIngredientGroupRightBelow(
 					const IngredientGroup &upperGroup);
@@ -378,7 +363,7 @@ private:
     void initTimeForTreat();
     void displayPauseMessage(bool display);
     void displayStartMessage(bool display);
-    void displayMessage(const char *msg);
+    void displayMessage(int row, const char *msg);
     void playSoundEffect(SoundMixer::Chunk &wb);
 
 
