@@ -1,8 +1,8 @@
-/*  $Id: IngredientSprite.h,v 1.3 2002/04/03 06:11:28 sarrazip Exp $
+/*  $Id: IngredientSprite.h,v 1.6 2004/05/02 04:55:48 sarrazip Exp $
     IngredientSprite.h - Sprite that is part of an "ingredient"
 
     burgerspace - A hamburger-smashing video game.
-    Copyright (C) 2001 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2001-2004 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -23,11 +23,13 @@
 #ifndef _H_IngredientSprite
 #define _H_IngredientSprite
 
-#include <gengameng/Sprite.h>
+#include <flatzebra/Sprite.h>
 
+#include <vector>
+
+using namespace flatzebra;
 
 class IngredientGroup;
-typedef vector<class IngredientSprite *> IngredientSpriteList;
 
 
 class IngredientSprite : public Sprite
@@ -35,6 +37,9 @@ class IngredientSprite : public Sprite
 */
 {
 public:
+
+    typedef std::vector<class IngredientSprite *> List;
+
     enum { LOWERING_DISTANCE = 4 };
 
     IngredientSprite(const PixmapArray &pixmapArray,
@@ -51,8 +56,10 @@ public:
     bool isLowered() const { return lowered; }
 
 private:
+
     IngredientGroup *myGroup;
     bool lowered;
+
 };
 
 
